#include <stdio.h>
#include <iostream>
#include <unistd.h>

#define C_KEY '<'
#define DECODE_MASK 0x7f
#define READ_BUF_SIZE 1024

using namespace std;

static int WriteCompressionFile(
	FILE *fp,
	unsigned char PreData,
	unsigned int ChainCount
)
{
  int i;
  if(fp==NULL){
    return false;
  }
  if(PreData==C_KEY){
    fprintf(fp,"%c%c",C_KEY,PreData);
    while(1){
      if(ChainCount/128>0){
	fprintf(fp,"%c",(unsigned char)((ChainCount%128)|128));
	ChainCount/=128;
      }else{
	fprintf(fp,"%c",(unsigned char)ChainCount);
	break;
      }
    }
  }else if(ChainCount==1){
    fprintf(fp,"%c",PreData);
  }else if(ChainCount==2 || ChainCount==3){
    for(i=0;i<ChainCount;i++){
      fprintf(fp,"%c",PreData);
    }
  }else{
    fprintf(fp,"%c%c",C_KEY,PreData);
    while(1){
      if(ChainCount/128>0){
	fprintf(fp,"%c",(unsigned char)((ChainCount%128)|128));
	ChainCount/=128;
      }else{
	fprintf(fp,"%c",(unsigned char)ChainCount);
	break;
      }
    }
  }
  return true;
}

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+ FUNC : JB_Comp
//+ 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
static int JB_Comp(FILE *fp_write,char *ReadBuf,int ReadSize){
  unsigned char Data;
  int i;
  static unsigned char PreData;
  static unsigned int ChainCount=0;

  if(ReadSize==0){
    WriteCompressionFile(fp_write,PreData,ChainCount);
    ChainCount=0;
    return true;
  }

  for(i=0;i<ReadSize;i++){
    Data=(unsigned char)ReadBuf[i];
    if(ChainCount==0){
      PreData=Data;
      ChainCount=1;
      continue;
    }

    if(PreData==Data){
      ChainCount++;
    }else{
      WriteCompressionFile(fp_write,PreData,ChainCount);

      ChainCount=1;
      PreData=Data;
    }
  }
  return true;
}

//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+ FUNC : JB_Compression
//+ 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
int JB_Compression(
	const char *readFileName,
	const char *writeFileName
){
  FILE *fp_read = stdin;
  FILE *fp_write = stdout;
  int ReadSize;
  char ReadBuf[READ_BUF_SIZE];

  if(readFileName){
  	if((fp_read=fopen(readFileName,"r"))==NULL){
    	return false;
  	}
  }

  if(writeFileName){
  	if((fp_write=fopen(writeFileName,"w"))==NULL){
		if(readFileName)
    		fclose(fp_read);
    	return false;
  	}
  }

  while(1){
    if((ReadSize=fread(ReadBuf,1,READ_BUF_SIZE,fp_read))==0){
      break;
    }
    JB_Comp(fp_write,ReadBuf,ReadSize);
  }
  JB_Comp(fp_write,NULL,0);

  if(readFileName)
  	fclose(fp_read);

  if(writeFileName)
  	fclose(fp_write);

  return true;
}

void usage()
{
	cout << "Usage: jbcomp [FILE] [OPTION]" << endl;
	cout << "Compress XSVF file sending to JTAG-Blazer" << endl;
	cout << "With no FILE, read standard input" << endl;
	cout << " -o,\toutput file name" << endl;
	cout << " -h,\tshow this message" << endl;
}

int main(int argc, char* argv[])
{
	int opt;
	char* outFileName = NULL;
	char* inFileName = NULL;
	while((opt = getopt(argc, argv, "ho:")) != -1){
		switch(opt){
		case 'o':
			outFileName = optarg;
			break;
		case 'h':
			usage();
			return 0;
		default:
			usage();
			return -1;
		}
	}
			
	if(optind < argc){
		if(optind < (argc - 1)){
			cerr << argv[0] << ": Invalid command format" << endl;
			usage();
			return -1;
		}
		inFileName = argv[optind];
	}
		
	if(!JB_Compression(inFileName, outFileName)){
		cerr << "Failed to compress file, please check permission" << endl;
		return -1;
	}
	return 0;
}
