#include <iostream>
#include <stdio.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <vector>
#include <unistd.h>
#include "jblib.h"

void usage()
{
	cout << "Usage: jbdevicelist [IP ADDRESS] [OPTION]" << endl;
	cout << "Show chained device ID connected to JTAG-Blazer" << endl;
	cout << " -h,\tshow this message" << endl;
}

using namespace std;

int main(int argc, char** argv)
{
	int opt;
	while((opt = getopt(argc, argv, "h")) != -1){
		switch(opt){
		case 'h':
			usage();
			return 0;
		default:
			usage();
			return -1;
		}
	}

	if(optind >= argc){
		cerr << argv[0] << ": IP address is not specified" << endl;
		usage();
		return -1;
	}
	in_addr addr;
	if(!inet_aton(argv[optind], &addr)){
        cerr << argv[0] << ": Invalid IP address" << endl;
        usage();
        return -1;
	}

	CJBLib::ErrorCode errCode;
	CJBLib lib(addr.s_addr);
	vector<unsigned int> deviceList;
	errCode = lib.DeviceListRequest(&deviceList);

	if(errCode == CJBLib::JTAG_NO_ERROR){
		int i;
		for(i = 0 ; i < deviceList.size() ; i++){
			printf("%3d %08x\n", i, deviceList[i]);
		}
	}
	else
		fprintf(stderr, "Failed (code : %08x)\n", (int)errCode);

	return (int)errCode;
}
