#include <iostream>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "jblib.h"

void usage()
{
    cout << "Usage: jbflash [IP ADDRESS] [FILE]" << endl;
    cout << "Write configuration file to JTAG-Blazer's flash memory" << endl;
    cout << " -h,\tshow this message" << endl;
    cout << " -c,\tcompressed file" << endl;
    cout << " -l,\tTE7720 mode" << endl;
}

void WriteProgress(int status, int param)
{
	if(status == 1){
		cout << "erasing..." << flush;
	}
	else{
		cout << "\rwrite size : " << param << flush;
	}
}

int main(int argc, char** argv)
{
	FlashFileData fileData;
	memset(&fileData, 0, sizeof(fileData));

	int opt;
	bool verbose = false;
	while((opt = getopt(argc, argv, "vhlc")) != -1){
		switch(opt){
		case 'v':
			verbose = true;
			break;
		case 'l':
			fileData.littleBear = true;
			break;
		case 'c':
			fileData.compressed = true;
			break;
		case 'h':
			usage();
			return 0;
		default:
			usage();
			return -1;
		}
	}

	int argPos = optind;
	if(argPos >= argc){
		cerr << argv[0] << ": IP address is not specified" << endl;
		usage();
		return -1;
	}
	in_addr addr;
	if(!inet_aton(argv[optind], &addr)){
		cerr << argv[0] << ": Invalid IP address" << endl;
		usage();
		return -1;
	}

	argPos++;
	if(argPos >= argc){
		cerr << argv[0] << ": FILE is not specified" << endl;
		usage();
		return -1;
	}
	const char* fileName = argv[argPos];

	char* pos = strrchr(fileName, '/');
	strcpy(fileData.fileName, pos? (pos + 1) : fileName);

	struct stat fstatus;
	if(stat(fileName, &fstatus)){
		perror(argv[0]);
		return -1;
	}
	fileData.fileSize = fstatus.st_size;
	fileData.recordTime = time(NULL);

	CJBLib lib(addr.s_addr);
	CJBLib::ErrorCode errCode = lib.FlashWriteRequest(
											fileName,
											&fileData,
											verbose? WriteProgress : NULL
											);

	if(verbose) cout << endl;
	if(errCode != CJBLib::JTAG_NO_ERROR)
		fprintf(stderr, "Failed (code : %08x)\n", (int)errCode);

	return (int)errCode;
}
