#include <iostream>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "jblib.h"

void usage()
{
	cout << "Usage: jbprogupdate [IP ADDRESS] [FILE]" << endl;
	cout << "Write firmware program to JTAG-Blazer" << endl;
	cout << " -h,\tshow this message" << endl;
}

void WriteProgress(int status, int param)
{
		cout << "\rwrote size : " << param;
}

int main(int argc, char** argv)
{
	int opt;
	bool verbose = false;
	while((opt = getopt(argc, argv, "vh")) != -1){
		switch(opt){
		case 'v':
			verbose = true;
			break;
		case 'h':
			usage();
			return -1;
		default:
			usage();
			return -1;
		}
	}

	int argPos = optind;
	if(argPos >= argc){
		cerr << argv[0] << ": IP address is not specified" << endl;
		usage();
		return -1;
	}
	in_addr addr;
	if(!inet_aton(argv[optind], &addr)){
		cerr << argv[0] << ": Invalid IP address" << endl;
		usage();
		return -1;
	}

	argPos++;
	if(argPos >= argc){
		cerr << argv[0] << ": FILE is not specified" << endl;
		usage();
		return -1;
	}
	const char* fileName = argv[argPos];

	CJBLib lib(addr.s_addr);
	CJBLib::ErrorCode errCode = lib.UpdateProgramRequest(
									fileName, verbose? WriteProgress : NULL);

	if(errCode != CJBLib::JTAG_NO_ERROR)
		fprintf(stderr, "Failed (code : %08x)\n", (int)errCode);

	return (int)errCode;
}
