//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//
//+    FILE : stj_process.h
//+ SUMMARY : SVFtoJSVF SVF process & JSVF process function header
//+     REV : 1.00 (2005/01/21)
//+  AUTHOR : M.Nakai (Atmark-Techno,Inc.)
//+    MEMO : 
//+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+*+//

#ifndef _STJ_PROCESS_H_
#define _STJ_PROCESS_H_

typedef struct{
  int  BitLen;
  unsigned char *TDI;
  unsigned char *TDO;
  unsigned char *MASK;
  unsigned char *SMASK;
}SVF_Trailer_Header;

typedef SVF_Trailer_Header SVF_Trailer;
typedef SVF_Trailer_Header SVF_Header;

#define MAX_TD_LEN 2043*8 //PR}hŏrbg
#if (MAX_TD_LEN<8)
#error MAX_TD_LEN
#endif
#define TD_BUF_SIZE (MAX_TD_LEN/8)

typedef struct{
  int BitLen;
  char TDI_Flag;
  char TDO_Flag;
  char MASK_Flag;
  char SMASK_Flag;
  DC_Position TDI_Pos;
  DC_Position TDO_Pos;
  DC_Position MASK_Pos;
  DC_Position SMASK_Pos;
  int TDI_CharLen;
  int TDO_CharLen;
  int MASK_CharLen;
  int SMASK_CharLen;
}CommandInfo;

typedef struct{
  unsigned char WaitState;
  unsigned char EndState;
  int Wait;
}InfoRUNTEST;

typedef enum _JSVF_COM{
  JSVF_COMPLETE    =0x00,
  JSVF_TDOMASK     =0x01,
  JSVF_SIR         =0x02,
  JSVF_SDR         =0x03,
  JSVF_RUNTEST     =0x04,
  JSVF_REPEAT      =0x07,
  JSVF_SDRSIZE     =0x08,
  JSVF_SDRTDO      =0x09,
  JSVF_SETSDRMASKS =0x0A,
  JSVF_SDRINC      =0x0B,
  JSVF_SDRB        =0x0C,
  JSVF_SDRC        =0x0D,
  JSVF_SDRE        =0x0E,
  JSVF_SDRTDOB     =0x0F,
  JSVF_SDRTDOC     =0x10,
  JSVF_SDRTDOE     =0x11,
  JSVF_STATE       =0x12,
  JSVF_ENDIR       =0x13,
  JSVF_ENDDR       =0x14,
  JSVF_SIR2        =0x15,
  JSVF_COMMENT     =0x16,
  JSVF_WAIT        =0x17,
  //JSVF_LAST_CMD    =0x18,
}JSVF_COM;

typedef enum _STATE_ID{
  STATE_RESET      =0x00,
  STATE_IDLE       =0x01,
  STATE_SELECT_DR  =0x02,
  STATE_CAPTURE_DR =0x03,
  STATE_SHIFT_DR   =0x04,
  STATE_EXIT1_DR   =0x05,
  STATE_PAUSE_DR   =0x06,
  STATE_EXIT2_DR   =0x07,
  STATE_UPDATE_DR  =0x08,
  STATE_SELECT_IR  =0x09,
  STATE_CAPTURE_IR =0x0a,
  STATE_SHIFT_IR   =0x0b,
  STATE_EXIT1_IR   =0x0c,
  STATE_PAUSE_IR   =0x0d,
  STATE_EXIT2_IR   =0x0e,
  STATE_UPDATE_IR  =0x0f,
}STATE_ID;

#define ENDIR_IDLE     0x00
#define ENDIR_PAUSE_IR 0x01

#define ENDDR_IDLE     0x00
#define ENDDR_PAUSE_DR 0x01


#define MODE_BASE  (0x20000000)

#define MASK_MODE1 (MODE_BASE | 0x00000003)
#define MODE_IR    (MODE_BASE | 0x00000001)
#define MODE_DR    (MODE_BASE | 0x00000002)

#define MASK_MODE2 (MODE_BASE | 0x000000F0)
#define MODE_TDI   (MODE_BASE | 0x00000010)
#define MODE_TDO   (MODE_BASE | 0x00000020)
#define MODE_MASK  (MODE_BASE | 0x00000040)
#define MODE_SMASK (MODE_BASE | 0x00000080)


#define RULE_BASE    (0x30000000)

#define RULE_RUNTEST (RULE_BASE | 0x00000001)

void DummyProcess(void);

int CreateJSVF(const char *file_name);
void ReleaseJSVF(void);
void WriteJSVF(const char *buf,const int size);

void JsvfSTATE(const unsigned char state);

void SvfProcessInitialize(const unsigned char repeat);
void SvfProcessEnd(void);

void SvfProcessRUNTEST(void);
void SvfProcessTRST(void);
void SvfProcessPIO(void);
void SvfProcessPIOMAP(void);
void SvfProcessFREQUENCY(void);
void SvfProcessENDIR(void);
void SvfProcessENDDR(void);
void SvfProcessSTATE(void);
void SvfProcessSIR(void);
void SvfProcessSDR(void);

void SvfProcessTIR(void);
void SvfProcessTDR(void);
void SvfProcessHIR(void);
void SvfProcessHDR(void);

void InitInfoRUNTEST(InfoRUNTEST *info);
void ScanRuntestInfo(InfoRUNTEST *info);
int RuleProcessRUNTEST(DC_Position *next,InfoRUNTEST *info,int flag);
int RuleProcessSDR(void);
#endif //_STJ_PROCESS_H_

